@@ -12,6 +12,8 @@ import re
 import shutil
 import sys
 
+from jinja2 import Environment
+from jinja2 import PackageLoader
 import py
 
 # We import our own defaults, these can be overridden.
@@ -20,20 +22,15 @@ from tha.taglist.defaults import BASE_ON_SERVER
 from tha.taglist.defaults import BLACKLIST
 from tha.taglist.defaults import STOP_INDICATORS
 from tha.taglist.defaults import OUTFILE
-from tha.taglist.defaults import HTMLSTART
-from tha.taglist.defaults import HTMLEND
-from tha.taglist.defaults import ENTRY
 
 DEFAULTS = ['BASE',
             'BASE_ON_SERVER',
             'BLACKLIST',
             'STOP_INDICATORS',
-            'OUTFILE',
-            'HTMLSTART',
-            'HTMLEND',
-            'ENTRY']
+            'OUTFILE']
 
 
+jinja_env = Environment(loader=PackageLoader('tha.taglist', 'templates'))
 logger = logging.getLogger('exporter')
 
 
@@ -196,11 +193,10 @@ def main(defaults_file=None, specific_outfile=None):
                         format="%(levelname)s: %(message)s")
     if defaults_file:
         override_global_constants(defaults_file)
-    handle, outfile_name = mkstemp(suffix='.html', prefix='taglist')
-    outfile = open(outfile_name, 'w')
-    logger.debug("Writing to tempfile %s", outfile_name)
+    template = jinja_env.get_template('index.html')
+
     today = datetime.now().ctime()
-    outfile.write(HTMLSTART % dict(date=today))
+
     if BASE_ON_SERVER:
         start = py.path.svnurl(BASE_ON_SERVER)
         if not start.check():
@@ -209,32 +205,21 @@ def main(defaults_file=None, specific_outfile=None):
     else:
         start = py.path.svnurl(BASE)
     latest = {}
+    entries = []
     for tagsdir in find_tag_directories(start):
         for tag, name, url in find_tags(tagsdir):
             logger.info("Found tag %s for %s: %s",
                         tag, name, url)
             update_latest(latest, tag, name, url)
-            outfile.write(ENTRY % dict(tag=tag, name=name, url=url))
-    # New: prepare a section with latest versions.  Ideally this needs some
-    # refactoring to use a proper Jinja template, but I've got another thingy
-    # that needs the current behaviour...
-    latest_part = ['<h2><a name="latest">Latest versions</a></h2>', '<dl>']
-    packages = latest.keys()
-    packages.sort()
-    for package in packages:
-        version = latest[package]
-        latest_part.append('<dt>%s</dt><dd>%s</dd>' % (package, version))
-    latest_part.append('</dl>')
-    outfile.write(HTMLEND % {'latest_versions': '\n'.join(latest_part)})
-    outfile.close()
+            entries.append(dict(tag=tag, name=name, url=url))
+
     target = os.path.join(os.getcwd(), OUTFILE)
     if specific_outfile:
         # For overriding from buildout.
         target = specific_outfile
-    shutil.move(outfile_name, target)
-    # ^^^ os.rename fails cross-filesystem.
-    # Maybe unnecessary, but on one system the resulting file wasn't
-    # readable for apache when created by another user. We'll just
-    # make it world readable.
+    content = template.render(today=today,
+                              entries=entries,
+                              latest=latest)
+    open(target, 'w').write(content)
     os.chmod(target, 0644)
     logger.info("Generated list written to %s", target)
